
#include "ds1620.h"

// DS1620
// Anschlsse CLK und DQ sind identisch
// Anschlsse des DS1620-1  (RST = B0, CLK = B1, DQ = B2) definiert in ds1620.h
// Anschlsse des DS1620-2  (RST = D7, CLK = B1, DQ = B2) definiert in ds1620.h

/* - intern ---- Set/Clear der RST-Leitung -------------------*/
// muss um die Anzahl der DS1620 erweitert werden (z.Zt. nur 2 DS1620)
void s_rst(int a)
{
	switch (a)
	{
		case 1: SET_RST;   break;
		case 2: SET_RST2;  break;
	};
};
void c_rst(int a)
{
	switch (a)
	{
		case 1: CLEAR_RST;   break;
		case 2: CLEAR_RST2;  break;
	};
};
/*----------------------------------------------------------------------*/
/*      Put temp from DS1620-2                                            */
/*----------------------------------------------------------------------*/
// sendet ein Byte an den DS1620-2 = 8 Bit

unsigned int GetTemp(int ds1620_nr)
{
	signed int temp;
	s_rst(ds1620_nr);
	Put1620byte(0xAA);
	temp=Read1620byte();
	temp = (temp*10)/2;			// Werte umrechnen 100=10,0 225=22,5
	c_rst(ds1620_nr);
	return temp;
}

void StartTemp(int ds1620_nr)
{
	OUTPUT;
	s_rst(ds1620_nr);
	Put1620byte(0xEE);
	c_rst(ds1620_nr);
};

/*
void StopTemp1(void)
{
	SET_RST;
	Put1620byte(0x22);
	CLEAR_RST;
}
*/

// ReadConfig ist scheinbar nicht IO
// Rckgabewerte mssen geprft werden
char ReadConfig1(void)					//IO
{
	char conf;
	SET_RST;
	Put1620byte(0xAC);
	conf=Get1620byte();
	CLEAR_RST;
	return conf;
};
char ReadConfig2(void)					//Fehler (??)
{
	char conf;
	SET_RST2;
	Put1620byte(0xAC);
	conf=Get1620byte();
	CLEAR_RST2;
	return conf;
};

void WriteConfig1(void)
{
	SET_RST;
	Put1620byte(0x0C);
	Put1620byte(0x03);		//CPU + 1Shot
	CLEAR_RST;
	return;

};
void WriteConfig2(void)
{
	SET_RST2;
	Put1620byte(0x0C);
	Put1620byte(0x03);		//CPU + Shot
	CLEAR_RST2;
	return;

};

/*----------------------------------------------------------------------*/
/*      Get temp from DS1620                                            */
/*----------------------------------------------------------------------*/

// Liest die Konfiguration des DS1620 = 8 Bit
char Get1620byte(void)
{
	unsigned char j,k=0,b=1;
	k=0;
	b=1;
	INPUT;
    	CLEAR_DQ;
//    	for (j=0; j<9; j++)
    	for (j=0; j<8; j++)
	{
		CLEAR_CLK;
//		DELAY_MS(3);
		if (bit_is_set(PINB, 2))
			k|=b;                                /* Read bit and or if = 1  */

		SET_CLK;
		//DELAY_MS(3);
		b=(b<<1);                            /* Setup for next read and or */
	}
	return k;
}

/*----------------------------------------------------------------------*/
/*      Put temp from DS1620                                            */
/*----------------------------------------------------------------------*/
// sendet ein Byte an den DS1620 = 8 Bit
// sollte Put1620Obyte1 und 2 ersetzen
void Put1620byte(char m)
{
	char k;
	OUTPUT;
//	SET_RST;

	for (k=0; k<8; k++)
//	for (k=0; k<9; k++)
	{
		CLEAR_CLK;
		if((m & 1))
			{	SET_DQ;		}
		else
			{	CLEAR_DQ;	};
		SET_CLK;
		m=(m>>1);                            // Setup to send next bit
	}
	INPUT;
	return;
};


/*----------------------------------------------------------------------*/
/*      read temp from DS1620                     fr jeden gleich                      */
/*----------------------------------------------------------------------*/

// Liest 9 Bit vom DS1620
unsigned int Read1620byte(void)
{
	unsigned char j,b=1;
	unsigned int k=0;
	INPUT;		//nur zur Vorsicht noch einmal (muss sein ??)
	k=0;
	b=1;
    	CLEAR_DQ;
    for (j=0; j<9; j++)
	{
		CLEAR_CLK;
		//DELAY_MS(1);
		// minimale Verzgerung ist erforderlich,
		// scheinbar ist die Flanke nicht sauber genug
 		sbi(PORTD,6);

 		if (bit_is_set(PINB, 2))
			k|=b;                           	// Read bit and or if = 1

		SET_CLK;
		//DELAY_MS(1);
		b=(b<<1);                            	// Setup for next read and or
	}
	return k;
}

/*----------------------------------------------------------------------*/
/*      write temp from DS1620                                          */
/*----------------------------------------------------------------------*/
//schreibt 9 Bit zum DS1620
void Write1620byte(char m)
{
	unsigned char k;
 	OUTPUT;
	SET_RST;
    	for (k=0; k<10; k++)
	{
		CLEAR_CLK;

		if((m & 1))
			SET_DQ;
		else
			CLEAR_DQ;

        	SET_CLK;

        	m=(m>>1);                            /* Setup to send next bit  */
	}
	INPUT;
	return;
}

/*
#define SET_CLK		{ sbi(PORTB,1); sbi(PORTD,6);}
#define CLEAR_CLK	{ cbi(PORTB,1); cbi(PORTD,6);}
#define SET_RST		sbi(PORTB,0)
#define CLEAR_RST	cbi(PORTB,0)
#define INPUT		cbi(DDRB,2)
#define OUTPUT		sbi(DDRB,2)
#define SET_DQ		sbi(PORTB,2)
#define CLEAR_DQ	cbi(PORTB,2)
*/

void DS1620ini (void)
{
	// DS1620-1
	sbi (DDRB,1);   // CLK output
	cbi (DDRB,2);	// DQ input
	sbi (DDRB,0);   // RST output  DS1620-1
	// DS1620-2
	sbi (DDRD,7);	// RST output DS1620-2
	SET_CLK;
	DELAY_MS(10);   //vermutlich nicht bentigt
	CLEAR_RST;
    CLEAR_RST2;
	CLEAR_DQ;

};
